/// <reference types="react" />
interface ClientSize {
    clientHeight?: number;
    clientWidth?: number;
}
interface DatasetLength {
    rowCount?: number;
    columnCount?: number;
}
interface ScrollSize {
    scrollHeight?: number;
    scrollWidth?: number;
}
interface ScrollPosition {
    scrollLeft?: number;
    scrollTop?: number;
}
declare enum ScrollAxisName {
    VERTICAL = "vertical",
    HORIZONTAL = "horizontal"
}
interface ScrollAxis<T> {
    [ScrollAxisName.VERTICAL]?: T;
    [ScrollAxisName.HORIZONTAL]?: T;
}
declare enum ScrollDirection {
    UP = "up",
    DOWN = "down",
    LEFT = "left",
    RIGHT = "right"
}
declare type ScrollDirectionState = Partial<Record<ScrollDirection, boolean>>;
declare type ScrollParams = ScrollSize & ScrollPosition & ClientSize;
declare type EventListenerFn = (type: 'scroll', callback: () => void) => void;
declare type SetRefFn = (ref: any) => void;
declare type ScrollingElementRef = Required<ScrollSize> & Required<ScrollPosition> & Required<ClientSize>;
declare type RegisterEventListener = {
    addEventListener: EventListenerFn;
    removeEventListener: EventListenerFn;
};
interface ScrollingContainerRef {
    scrollingElement: ScrollingElementRef | null;
    registerEventListener: RegisterEventListener | null;
}
interface VirtualizedScrollingContainer {
    _scrollingContainer: HTMLElement;
}
interface VirtualizedGrid {
    Grid: VirtualizedScrollingContainer;
}
declare type ElementRef = VirtualizedScrollingContainer | VirtualizedGrid | HTMLElement;
declare type InfiniteScrollState = DatasetLength & Required<ScrollSize> & Required<ClientSize> & {
    isLoading: boolean;
    thresholdReached: ScrollDirectionState;
    computedScrollThreshold: Required<ScrollAxis<number>>;
    cleanup: (() => void)[];
};
declare type UseInfiniteScrollProps = DatasetLength & {
    hasMore: ScrollDirectionState;
    next: (direction: ScrollDirection) => Promise<void>;
    onScroll?: (value: Required<ScrollParams>) => void;
    scrollThreshold?: number | string;
    initialScroll?: {
        top?: number;
        left?: number;
    };
    reverse?: {
        column?: boolean;
        row?: boolean;
    };
    windowScroll?: boolean;
};
declare type InfiniteScrollRef<T> = React.MutableRefObject<T | null>;
declare type InfiniteScrollProps = UseInfiniteScrollProps;
declare type ScrollOffsetValues = Required<ScrollDirectionState>;

declare const useInfiniteScroll: <T>(props: UseInfiniteScrollProps) => InfiniteScrollRef<T>;

declare class InfiniteScroll {
    props: InfiniteScrollProps;
    state: InfiniteScrollState;
    _scrollingContainerRef?: ScrollingContainerRef;
    constructor(props: InfiniteScrollProps);
    _validateScrollValue: (scrollPosition: number, scrollSize: number, fallbackValue?: number) => number;
    _scroll: (this: InfiniteScroll, { scrollTop, scrollLeft }: ScrollPosition) => void;
    _computeThreshold: (this: InfiniteScroll) => void;
    _getPossibleDirections: (this: InfiniteScroll, offset?: number) => Required<ScrollOffsetValues>;
    _loadByDirection: (this: InfiniteScroll, direction1: ScrollDirection.UP | ScrollDirection.LEFT, direction2: ScrollDirection.DOWN | ScrollDirection.RIGHT, offset: Required<ScrollOffsetValues>) => Promise<void>;
    _checkOffsetAndLoadMore: (this: InfiniteScroll) => void;
    _setRef: (this: InfiniteScroll, ref: ElementRef | null) => void;
    _onLoadComplete: (this: InfiniteScroll, axis: ScrollAxisName) => void;
    _onPropsChange: (this: InfiniteScroll, props: UseInfiniteScrollProps) => void;
    _onCleanup: (this: InfiniteScroll) => void;
    setRef: (ref: ElementRef | null) => void;
    onPropsChange: (props: UseInfiniteScrollProps) => void;
    onCleanup: () => void;
}

export { ClientSize, DatasetLength, ElementRef, EventListenerFn, InfiniteScroll, InfiniteScrollProps, InfiniteScrollRef, InfiniteScrollState, RegisterEventListener, ScrollAxis, ScrollAxisName, ScrollDirection, ScrollDirectionState, ScrollOffsetValues, ScrollParams, ScrollPosition, ScrollSize, ScrollingContainerRef, ScrollingElementRef, SetRefFn, UseInfiniteScrollProps, VirtualizedGrid, VirtualizedScrollingContainer, useInfiniteScroll as default, useInfiniteScroll };
